#ifndef SEZNAM_T_H
#define SEZNAM_T_H

//seznam_t.h - sablona pro neintruzivni DDS seznam (2.12.1999)

/*
 * 0. Obecne - komentar
 *
 * 1. Definice prvku seznamu
 *
 * 2. Definice seznamu
 *
 * 3. Definice iteratoru seznamu
 *
 * 4. Definice vyjimecnych stavu
 *
 */

//====================0. OBECNE O IMPLEMENTACI SEZNAMU========================

/*
 *
 * - seznam vypada takto:
 *                           p_object   +-+          p_object   +-+
 *                             +------->|T|            +------->|T|
 *                             |        +-+            |        +-+
 *                 +---------------+         +---------------+
 *    NULL <-------|seznam_prvek<T>|<--------|seznam_prvek<T>|
 *          p_pred |               | p_pred  |               |
 *                 |               |-------->|               |--------> NULL
 *            +--->+---------------+ p_dalsi +---------------+ p_dalsi
 *            |                                       ^
 *         p_zacatek                                  |
 *                                                    |
 *                                                 p_konec
 *
 * - seznam je napsan v ANSI C++
 *
 * - jde o obecne napsany kontejner pro libovolnou tridu a jeji potomky, tato
 *   vyhodna vlastnost (ukladani potomku a zajisteni spravne manipulace
 *   s nimi) sebou nese i jiste omezeni:
 *          - v seznamu mohou byt jen uzivatelsky definovane typy, protoze
 *            seznam klade urcite naroky na typ, ktery do nej chceme vlozit
 *          - trida musi mit urcite (snadno imlementovatelne) vlastnosti
 *
 * - naroky na tridu, ktera ma byt v seznamu:
 *          - MUSI mit spravny kopirovaci konstruktor:
 *                 X(const& X)
 *          - NENI NUTNE aby mela prirazovaci operator:
 *                 const& X operator=(const& X)
 *          - MUSI mit porovnavaci operator:
 *                 int operator==(const X&)
 *                 - pocita s tim jedna fce
 *                 - alespon formalni definice - prazdna metoda - aby
 *                   kompilator nehlasil chybu - viz napr. trida seznam_t nize
 *          - NENI NUTNE aby mela i opacny operator - tj. nerovnosti:
 *                 int operator!=(const X&)
 *          - MUSI mit klonovaco operaci ve tvaru:
 *                 virtual zakladova_trida* klon()
 *                   { return new odvozena_trida(*this); }
 *
 * - vsechny prvky seznamu jsou ulozeny ve volne pameti (zatim je podporovano
 *   jen vkladani pointeru)
 *
 * - kazdy prvek krome vazebnich polozek nese indikaci, zda je prave
 *   pouzivan (takovy prvek potom nemuze byt rusen jinou fci, nez tou, ktera
 *   s nim prave pracuje apod. - vice viz seznam_prvek_t::zamceno)
 *
 */

//====================1. PRVEK SEZNAMU========================================

/*
 * Prvek seznamu v sobe uchovava:
 *       (1) Hodnotu (pointer na hodnotu), ktera je ulozena v seznamu.
 *       (2) Vazebni polozku - pointer na dalsi + predchazejici prvek.
 *       (3) "Indikator" pouziti - viz nize.
 *
 * Seznam_prvek (a nasledne i seznam a seznam_iterator) jsou zalozeny na
 * predpokladu, ze objektym ktere seznam uchovava jsou ve volne pameti.
 * Tomu tudiz odpovida konstruktor:
 *      a) pokud se predava pointer na objekt, predpoklada se jiz, ze objekt
 *         ve volne pameti je (pouze se nastavi p_object).
 *      b) je-li predavan odkaz na objekt, je pridelena pamet pomoci
 *         new a objekt je prekopirovan.
 *
 * Vsechny cleny jsou private, aby se zajistilo, ze seznam_prvek_t<T> bude
 * vytvaret a rusit pouze seznam_t<T>, resp. seznam_iterator_t<T>. Jedina
 * vyjimka - fce pro zamykani a odemykani - viz nize.
 *
 * Pri ruseni prvku seznamu se rusi zaroven objekt, na ktery prvek ukazuje
 * (tj. seznam_prvek::p_object).
 *
 * "Indikator" pouziti - seznam_prvek_t::zamceno - predpoklada nasledujici
 * pouziti: Kazda fce, ktera s prvkem pracuje (cte data, meni data, ...
 * objektu) zvysi tuto promenou o 1. Prvek nemuze byt zrusen, pokud s prvkem
 * (resp. objektem) pracuje vice fci nez 1 (tj. seznam_prvek::zamceno > 0).
 * Pochopitelne, ze pri ukonceni prace s prvkem je tato promnena zase o 1
 * zmensena.
 *
 */

//dopredne deklarace
template <class T> class seznam_t;
template <class T> class seznam_iterator_t;

template <class T> class seznam_prvek_t {
  seznam_prvek_t<T>* p_dalsi;
  seznam_prvek_t<T>* p_pred;
  T* p_object;
  signed int zamceno;

  //konstrukce, destrukce
  seznam_prvek_t(T* p_T)
    : p_dalsi(0), p_pred(0), zamceno(-1), p_object(p_T) { }
  seznam_prvek_t(const T& r_T)
    : p_dalsi(0), p_pred(0), zamceno(-1) { p_object = r_T.klon(); }
  seznam_prvek_t(const seznam_prvek_t<T>& r_sp)
    : p_dalsi(0), p_pred(0), zamceno(-1)
    { p_object = r_sp.p_object->klon(); }
  ~seznam_prvek_t() { delete p_object; }

  //spratelene tridy
  friend class seznam_t<T>;
  friend class seznam_iterator_t<T>;

};

//==============================2. SEZNAM=====================================

/*
 * Seznam podporuje zakladni operace nad prvky - vkladani/vybirani prvku na
 * konec/zacatek. Vedle toho disponuje i par dalsimi fcemi (vymaz_vse(), aj.).
 * Seznam nepodporuje princip aktualniho prvku - viz seznam_iterator.
 *
 * Operace vkladani a vybirani jsou definovany dvojmo - pro hodnotu a pointer;
 * duvody, ktere k tomu vedou viz seznam_prvek.
 *
 * Pri operacich vybirani objektu (tj. objektu ulozenych v retezenych
 * seznam_prvcich) vraci fce pointer na objekt (nove vytvoreny ve volne
 * pameti), pricemz seznam_prvek (vcetne objektu na ktery ukazuje) je zrusen.
 * Pri vyberu z prazdneho seznamu nebo zamceneho prvku je vracen NULL.
 *
 * Vybrany prvek je umisten ve volne pameti a tudiz bude nutne ho nekdy
 * zrusit.
 *
 * Pri destrukci seznamu jsou ruseny vsechny jeho prvky a tudiz i objekty,
 * jez obsahuje.
 *
 */

template <class T> class seznam_t {
protected:
  seznam_prvek_t<T>* p_zacatek;
  seznam_prvek_t<T>* p_konec;
public:
  //zakladni operace
  seznam_t() { p_zacatek = p_konec = 0; }
  seznam_t(const seznam_t& r_s);
  virtual seznam_t<T>* klon() const
    { return new seznam_t<T>(*this); }
  virtual ~seznam_t();

  //operace vkladani a vybirani
  void na_zacatek(T* p_T);
  void na_konec(T* p_T);
  T* vyber_zacatek();
  T* vyber_konec();
  T* vyber_nty(const unsigned int i);
  T* vyber_dany(const T& vzor);
  void vyber_vse();

  //ostatni
  T* zacatek() const
    { return (p_zacatek) ? p_zacatek->p_object : 0; }
  T* konec() const { return (p_konec) ? p_konec->p_object : 0; }
  T* operator[](const unsigned int index) const;
  T* operator[](const T& vzor) const;
  int pocet_prvku() const;
  int operator==(const seznam_t&) { return 0; } //formalni definice
  int operator!=(const seznam_t&) { return 0; } // -"-

  //spratelene tridy
  friend class seznam_iterator_t<T>;
};

//----------------------SEZNAM_T<T>::SEZNAM(CONST SEZNAM_T<T>&)---------------
template <class T> seznam_t<T>::seznam_t(const seznam_t<T>& r_s)
{
  if (r_s.p_zacatek) {
    p_zacatek = p_konec = new seznam_prvek_t<T>(*r_s.p_zacatek);
    for (seznam_prvek_t<T>* p_prvek = r_s.p_zacatek->p_dalsi;
	 p_prvek; p_prvek = p_prvek->p_dalsi) {
      p_konec->p_dalsi = new seznam_prvek_t<T>(*p_prvek);
      p_konec->p_dalsi->p_pred = p_konec;
      p_konec = p_konec->p_dalsi;
    }
  } //konec if (r_s.p_zacatek)
  else p_zacatek = p_konec = 0;
}

//--------------------------SEZNAM_T<T>::~SEZNAM_T()--------------------------
template <class T> seznam_t<T>::~seznam_t()
{
  while (p_zacatek) {
    seznam_prvek_t<T>* p_prvek = p_zacatek;
    p_zacatek = p_zacatek->p_dalsi;
    delete p_prvek;
  }
}

//-------------------------SEZNAM_T<T>::NA_ZACATEK(T*)------------------------
template <class T> void seznam_t<T>::na_zacatek(T* p_T)
{
  if (p_zacatek) {
    p_zacatek->p_pred = new seznam_prvek_t<T>(p_T);
    p_zacatek->p_pred->p_dalsi = p_zacatek;
    p_zacatek = p_zacatek->p_pred;
  }
  else p_zacatek = p_konec = new seznam_prvek_t<T>(p_T);
}

//-------------------------SEZNAM_T<T>::NA_KONEC(T*)--------------------------
template <class T> void seznam_t<T>::na_konec(T* p_T)
{
  if (p_konec) {
    p_konec->p_dalsi = new seznam_prvek_t<T>(p_T);
    p_konec->p_dalsi->p_pred = p_konec;
    p_konec = p_konec->p_dalsi;
  }
  else p_zacatek = p_konec = new seznam_prvek_t<T>(p_T);
}

//-------------------------SEZNAM_T<T>::VYBER_ZACATEK()-----------------------
template <class T> T* seznam_t<T>::vyber_zacatek()
  //zohleduje seznam_prvek_t<T>::zamceno
{
  if (p_zacatek) { //je co vybrat
    if (p_zacatek->zamceno == -1) { //prvek neni zamcen
      T* p_T = p_zacatek->p_object->klon();
      seznam_prvek_t<T>* p_prvek = p_zacatek;
      if (p_zacatek == p_konec) { //vyber z jednoprvkoveho seznamu
        p_zacatek = p_konec = 0;
      }
      else { //vyber z viceprvkoveho seznamu
        p_zacatek = p_zacatek->p_dalsi;
        p_zacatek->p_pred = 0;
      }
      delete p_prvek;
      return p_T;
    }
  }
  return 0;
}

//-------------------------SEZNAM_T<T>::VYBER_KONEC()-------------------------
template <class T> T* seznam_t<T>::vyber_konec()
  //zohleduje seznam_prvek_t<T>::zamceno
{
  if (p_konec) { //je co vybrat
    if (p_konec->zamceno == -1) { //prvek neni zamcen
      T* p_T = p_konec->p_object->klon();
      seznam_prvek_t<T>* p_prvek = p_konec;
      if (p_zacatek == p_konec) { //vyber z jednoprvkoveho seznamu
        p_zacatek = p_konec = 0;
      }
      else { //vyber z viceprvkoveho seznamu
        p_konec = p_konec->p_pred;
        p_konec->p_dalsi = 0;
      }
      delete p_prvek;
      return p_T;
    }
  }
  return 0;
}

//-------------------SEZNAM_T<T>::VYBER_NTY(CONST UNSIGNED INT)---------------
template <class T>  T* seznam_t<T>::vyber_nty(const unsigned int i)
  //vybere n-ty prvek seznamu, indexace jako pole (prvni -> i == 0)
  //zohlednuje seznam_prvek_t<T>::zamceno
{
  return 0;
}

//-------------------------SEZNAM_T<T>::VYBER_DANY(CONST T&)------------------
template <class T> T* seznam_t<T>::vyber_dany(const T& r_T)
  //predpoklada existenci porovnavaciho operatoru pro T
  //zohlednuje seznam_prvek_t<T>::zamceno
{
  seznam_prvek_t<T>* p_prvek = p_zacatek;
  while (p_prvek) { //sekvencni hledani
    if (*p_prvek->p_object == r_T && p_prvek->zamceno == -1) { //prvek se
                                                               //nasel a neni
                                                               //zamcen
      T* p_T = p_prvek->p_object->klon();
      if (p_zacatek == p_konec) { //jednoprvkovy seznam
        p_zacatek = p_konec = 0;
      }
      else if (p_prvek == p_zacatek) { //vyber zacatku z viceprvkoveho
        p_zacatek = p_zacatek->p_dalsi;
        p_zacatek->p_pred = 0;
      }
      else if (p_prvek == p_konec) { //vyber konce z viceprvkoveho
        p_konec = p_konec->p_pred;
        p_konec->p_dalsi = 0;
      }
      else { //vyber odnekud zprostredka
        p_prvek->p_pred->p_dalsi = p_prvek->p_dalsi;
	p_prvek->p_dalsi->p_pred = p_prvek->p_pred;
      }
      delete p_prvek;
      return p_T;
    } // konec if (*p_prvek->p_object == r_T && p_prvek->zamceno == -1)
    p_prvek = p_prvek->p_dalsi;
  } //konec while (p_prvek)
  return 0;
}

//--------------------------SEZNAM<T>::VYBER_VSE()----------------------------
template <class T> void seznam_t<T>::vyber_vse()
  //nezohlednuje zmaceno - rusi vse a vzdy
{
  while (p_zacatek) {
    seznam_prvek_t<T>* p_prvek = p_zacatek;
    p_zacatek = p_zacatek->p_dalsi;
    delete p_prvek;
  }
  p_zacatek = p_konec = 0;
}

//--------------------------SEZNAM<T>::OPERATOR[](INT)------------------------
template <class T> T* seznam_t<T>::operator[](const unsigned int index) const
  //adresace jako pole
{
  if (index < pocet_prvku()) {
    int i = 0;
    seznam_prvek_t<T>* p_prvek = p_zacatek;
    while (i < index) {
      p_prvek = p_prvek->p_dalsi;
      ++i;
    }
    return p_prvek->p_object;
  } //konec if (index < pocet_prvku())
  else return 0;
}

//--------------------------SEZNAM<T>::OPERATOR[](CONST T&)-------------------
template <class T> T* seznam_t<T>::operator[](const T& r_T) const
  //predpoklada existenci porovnavaciho operatoru pro T
{
  seznam_prvek_t<T>* p_prvek = p_zacatek;
  while (p_prvek) { //sekvencni hledani
    if (*p_prvek->p_object == r_T) return p_prvek->p_object;
    p_prvek = p_prvek->p_dalsi;
  }
  return 0;
}

//---------------------------SEZNAM<T>::POCET_PRVKU()-------------------------
template <class T> int seznam_t<T>::pocet_prvku() const
{
  seznam_prvek_t<T>* p_prvek = p_zacatek;
  int i = 0;
  while (p_prvek) {
    ++i;
    p_prvek = p_prvek->p_dalsi;
  }
  return  i;
}

//=======================3. ITERACE SEZNAMU===================================

/*
 * Iterator slouzi zejmena k prochazeni seznamu.
 *
 * Iteraci zajistuje metoda seznam_iterator_t::operator()(), ktera vraci vzdy
 * nasledujici objekt v seznamu (tj. ten, na ktery ukazuje
 * seznam_prvek_t::p_object) - vsimnete si, ze uzivatel seznamu a iteratoru
 * je zcela chranen pred existenci tridy seznam_prvek_t<T>.
 *
 * Konec iterace (po vraceni posledniho prvku seznamu) se pozna podle vraceni
 * NULL. Pokud budete behem iterace na konec neustale pridavat nove prvky,
 * iterace nikdy neskonci.
 *
 * Aktualni prvek (krome toho, ze je navracen clenskou fci operator()())
 * muze byt vracen i pomoci metody aktualni(). Vsimnete si, ze tato fce vraci
 * podobne jako seznam_t::zacatek() a seznam_t::konec() pointer na
 * seznam_prvek_t::p_object.
 *
 * Pruchod seznamem tedy muze vypadat takhle jednoduse:
 *
 *         seznam_t<T> xxx;
 *         //...
 *         seznam_iterator_t<T> iter(xxx);
 *         while (iter()) {
 *           iter.aktualni()->nejaka_fce();
 *           //...
 *         }
 *
 * Iterator se muze vratit do pocatecni pozice (tj. jako pred zapocetim
 * iterace) pomoci clenske fce obnov().
 *
 * Iterator podporuje princip aktualniho prvku. Lze tedy vkladat za / pred
 * aktualni prvek a vybirat aktualni prvek (vloz_za() / vloz_pred() a
 * vyber_aktualni()). Vybrat lze pochopitelne jen nezamceny prvek (tj.
 * nepouzivany). Fce vybirani a vkladani maji podobnou logiku fungovani
 * jako fce vkladani/vybirani u seznamu (tj. vraceni NULL pri pokusu vybrat
 * zamceny nebo neexistujici apod.).
 *
 * Pri vkladani a vybirani se meni pozice seznam_iterator_t::p_aktualni
 * nasledovne:
 *            1. Pri vlozeni pred aktualni se pozice p_aktualni nemeni.
 *            2. Pri vlozeni za aktualni se pozice p_aktualni nemeni. Prave
 *               vlozeny prvek bude vracen pri pristim kroku iterace.
 *            3. Pri vybrani aktualniho objektu se p_aktualni nastavi na
 *               predchazejici prvek.
 *
 * Vyuziti principu aktualniho prvku vyzaduje od uzivatele dodrzovani urcitych
 * pravidel - vyhnete se tak ruznym neprijemnostem. Kriticka operace je
 * vyjmuti prvku (vyber_aktualni()). Napr. pokud budete vybirat v jednom kroku
 * iterace dvakrat aktualni prvek a ten zrovna bude ukazovat na prvni prvek
 * seznamu - podle definice se posune aktualni na predchozi - bohuzel v tomto
 * pripade je aktualni nastaven na NULL - zadny predchozi totiz neni - druhy
 * pokus o vyber vrati NULL (iterator se bude domnivat ze neni v aktivnim
 * stavu). Ovsem toto je zcela specificky pripad. Vybirani samozrejme funguje
 * bezproblemove, pokud je uvazene uzivano.
 *
 * Suma sumarum jde o to, pouzivat iteraci rozumne:
 * Iterator vrati aktualni prvek, neco s nim provedu, a jedu dal, iterator se
 * stara o posunuti a vraceni, vy jen o tu praci - na teto logice je iterator
 * vystaven a tak take funguje. Mate-li specificke pozadavky, musite to
 * nejprve vyzkouset a pak si pripadne napsat svuj vlastni iteraci. Nyni ta
 * doporuceni:
 *            1. Iterator bude aktivni (tj. p_akt != NULL).
 *            2. Zejmena vybrani pouzivejte jednou za krok iterace. U vlozeni
 *               uz tak opatrni byt nemusite, jen pozor na vecnou iteraci!
 *            3. Behem iterace muzete bez problemu uzivat fce, ktere poskytuje
 *               trida seznam. Jsou napsany tak, aby zohlednili zamceni prvku.
 *
 */

template <class T> class seznam_iterator_t {
  seznam_prvek_t<T>* p_akt; //aktualni prvek
  seznam_t<T>& r_seznam;
public:
  //zakladni operace
  seznam_iterator_t(seznam_t<T>& r_s)
    : p_akt(0), r_seznam(r_s) { }

  //iterace
  T* operator()();

  //princip aktualniho prvku
  void za_aktualni(T* p_T);
  void pred_aktualni(T* p_T);
  T* vyber_aktualni();

  //ostatni
  T* aktualni() const { return (p_akt) ? p_akt->p_object : 0; }
  void obnov() const;
};

//------------------SEZNAM_ITERATOR_T<T>::OPERATOR()()------------------------
template <class T> T* seznam_iterator_t<T>::operator()()
{
  if (r_seznam.p_zacatek) { //je co vracet
    if (p_akt) { //v dalsich krocich iterace
      --p_akt->zamceno; // odemknuti prvku, ktery byl zamcen timto iteratorem
      p_akt = p_akt->p_dalsi;
      if (p_akt) ++p_akt->zamceno; //nastaveni zamku na (nyni jiz) aktualnim
      return (p_akt) ? p_akt->p_object : 0;
    }
    else { //v prvnim kroku iterace
      p_akt = r_seznam.p_zacatek;
      ++p_akt->zamceno;
      return p_akt->p_object;
    }
  }
  return 0;
}

//-------------------SEZNAM_ITERATOR_T<T>::ZA_AKTUALNI(T* p_T)----------------
template <class T> void seznam_iterator_t<T>::za_aktualni(T* p_T)
{
  if (r_seznam.p_zacatek) { //v seznamu jsou prvky a iterator je aktivni
    if (p_akt == r_seznam.p_konec) { //vkladam na konec seznamu
      r_seznam.p_konec->p_dalsi = new seznam_prvek_t<T>(p_T);
      r_seznam.p_konec->p_dalsi->p_pred = r_seznam.p_konec;
      r_seznam.p_konec = r_seznam.p_konec->p_dalsi;
    }
    else if (!p_akt) { //tato moznost nastne jen ve zcela specifickych
                       //pripadech (napr. kdyz vyberete zacatek a ve stejnem
                       //kroku iterace za nej chcete neco vlozit)
                       //jde vlastne o vlozeni na zacatek (nebezpecne, pokud
                       //iterator jeste neni aktivni - proste vlozim prvek bez
                       //ohledu na cokoliv - nemam moznost to otestovat)
      r_seznam.p_zacatek->p_pred = new seznam_prvek_t<T>(p_T);
      r_seznam.p_zacatek->p_pred->p_dalsi = r_seznam.p_zacatek;
      r_seznam.p_zacatek = r_seznam.p_zacatek->p_pred;
    }
    else { //vkladam nekam mezi prvky
      seznam_prvek_t<T>* p_prvek = new seznam_prvek_t<T>(p_T);
      p_prvek->p_dalsi = p_akt->p_dalsi;
      p_prvek->p_pred = p_akt;
      p_akt->p_dalsi = p_prvek;
      p_prvek->p_dalsi->p_pred = p_prvek;
    }
  }
}

//-------------------SEZNAM_ITERATOR_T<T>::PRED_AKTUALNI(T* p_T)--------------
template <class T> void seznam_iterator_t<T>::pred_aktualni(T* p_T)
{
  if (p_akt) { //v seznamu jsou prvky a iterator je aktivni
    if (p_akt == r_seznam.p_zacatek) { //vkladam na zacatek seznamu
      r_seznam.p_zacatek->p_pred = new seznam_prvek_t<T>(p_T);
      r_seznam.p_zacatek->p_pred->p_dalsi = r_seznam.p_zacatek;
      r_seznam.p_zacatek = r_seznam.p_zacatek->p_pred;
    }
    else { //vkladam nekam mezi prvky
      seznam_prvek_t<T>* p_prvek = new seznam_prvek_t<T>(p_T);
      p_prvek->p_dalsi = p_akt;
      p_prvek->p_pred = p_akt->p_pred;
      p_akt->p_pred = p_prvek;
      p_prvek->p_pred->p_dalsi = p_prvek;
    }
  }
}

//-------------------SEZNAM_ITERATOR_T<T>::VYBER_AKTUALNI()-------------------
template <class T> T* seznam_iterator_t<T>::vyber_aktualni()
  //zohlednuje zamceno - pokud je prvek zamcen, je vracen NULL a neni vybran
{
  if (p_akt && !p_akt->zamceno) { //v seznamu jsou prvky a iterator je aktivni
                                  //a vybirany prvek neni zamceno (zam. == 0)
    seznam_prvek_t<T>* p_prvek = p_akt;
    T* p_T = p_prvek->p_object->klon();
    if (r_seznam.p_zacatek == r_seznam.p_konec) { //jednoprvkovy seznam
      p_akt = r_seznam.p_zacatek = r_seznam.p_konec = 0;
    }
    else if (p_akt == r_seznam.p_zacatek) { //vybiram zacatek z viceprvkoveho
      r_seznam.p_zacatek = r_seznam.p_zacatek->p_dalsi;
      p_akt = r_seznam.p_zacatek->p_pred = 0;
    }
    else if (p_akt == r_seznam.p_konec) { //vybiram konec z viceprvkoveho
      p_akt = r_seznam.p_konec = r_seznam.p_konec->p_pred;
      ++p_akt->zamceno; //musim nastavit zamek! (pri dalsim kroku iterace bych
                        //snizoval hodnotu zamceno, aniz bych ji snizil
      r_seznam.p_konec->p_dalsi = 0;
    }
    else { //vybiram odnekud zprostredka viceprvkoveho
      p_prvek->p_dalsi->p_pred = p_akt = p_akt->p_pred;
      ++p_akt->zamceno; //musim nastavit zamek!
      p_akt->p_dalsi = p_prvek->p_dalsi;
    }
    delete p_prvek;
    return p_T;
  }
  return 0;
}

//-------------------SEZNAM_ITERATOR_T<T>::OBNOV()----------------------------
template <class T> void seznam_iterator_t<T>::obnov() const
{
  if (p_akt) --p_akt->zamceno; //odemknuti mozneho zamceneho prvku
  ((seznam_iterator_t<T>*)this)->p_akt = 0; //odtypovani konstanty
}

//=======================4. DEFINICE VYJIMECNYCH STAVU========================

/*
 * Behem obsluhy seznamu muze dojit k nasledujicim stavum:
 *        A. Ve tride seznam
 *           1. Vlozeni do/vybrani z prazdneho seznamu -> navrat NULL.
 *           2. Vlozeni do/vybrani z plneho seznamu (jedno-, viceprvkoveho)
 *              -> OK/navrat objektu seznam_prvek::p_object ve volne pameti,
 *              odstraneni seznam_prvku ze seznamu.
 *           3. Vlozeni do/vybrani z iterujiciho seznamu (tj. hrozi, ze prvek
 *              bude pouzivan) -> OK.
 *        B. Ve tride seznam_iterator
 *           1. Iterace na prazdnem seznamu -> navrat NULL.
 *           2. Iterace na plnem/iterujicim seznamu - operace vkladani -> OK,
 *              za predpokladu, ze p_aktualni != 0
 *           3. Iterace na plnem/iterujicim seznamu - operace vyber_aktualni()
 *              ->  ?????
 *
 */

/*
class seznam_vyjimka { };

//----------------DEFAULT_SEZNAM_HANDLER--------------------------------------
void default_seznam_handler(const char* s, ostream& out = cerr)
  //vypisuje chybove hlaseni 's' do daneho proudu (implicitne cerr)
{
  out << s << '\n';
}

//---------------SET_SEZNAM_HANDLER-------------------------------------------
void set_seznam_handler(void (* p)(const char*, ostream&))
{
  seznam_handler = p;
}
*/

#endif //SEZNAM_T_H